@@ -1,32 +1,32 @@
 /* inputting files to be patched */
 
-/* $Id: inp.c,v 1.18 1997/07/21 17:59:46 eggert Exp $ */
+/* $Id: inp.c,v 1.21 1999/08/30 06:20:08 eggert Exp $ */
 
-/*
-Copyright 1986, 1988 Larry Wall
-Copyright 1991, 1992, 1993, 1997 Free Software Foundation, Inc.
+/* Copyright 1986, 1988 Larry Wall
+   Copyright 1991, 1992-1993, 1997-1998, 1999 Free Software Foundation, Inc.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; see the file COPYING.
-If not, write to the Free Software Foundation,
-59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-*/
+   You should have received a copy of the GNU General Public License
+   along with this program; see the file COPYING.
+   If not, write to the Free Software Foundation,
+   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #define XTERN extern
 #include <common.h>
 #include <backupfile.h>
 #include <pch.h>
+#include <quotearg.h>
 #include <util.h>
+#include <xalloc.h>
 #undef XTERN
 #define XTERN
 #include <inp.h>
@@ -55,17 +55,24 @@ static void too_many_lines PARAMS ((char const *)) __attribute__((noreturn));
 /* New patch--prepare to edit another file. */
 
 void
-re_input()
+re_input (void)
 {
     if (using_plan_a) {
-	free (i_buffer);
-	free (i_ptr);
+      if (i_buffer)
+	{
+	  free (i_buffer);
+	  i_buffer = 0;
+	  free (i_ptr);
+	}
     }
     else {
 	close (tifd);
 	tifd = -1;
-	free(tibuf[0]);
-	tibuf[0] = 0;
+	if (tibuf[0])
+	  {
+	    free (tibuf[0]);
+	    tibuf[0] = 0;
+	  }
 	tiline[0] = tiline[1] = -1;
 	tireclen = 0;
     }
@@ -74,34 +81,31 @@ re_input()
 /* Construct the line index, somehow or other. */
 
 void
-scan_input(filename)
-char *filename;
+scan_input (char *filename)
 {
     using_plan_a = ! (debug & 16) && plan_a (filename);
     if (!using_plan_a)
 	plan_b(filename);
-    switch (verbosity)
+
+    if (verbosity != SILENT)
       {
-      case SILENT:
-	break;
+	filename = quotearg (filename);
 
-      case VERBOSE:
-	say ("Patching file `%s' using Plan %s...\n",
-	     filename, using_plan_a ? "A" : "B");
-	break;
-
-      case DEFAULT_VERBOSITY:
-	say ("patching file `%s'\n", filename);
-	break;
+	if (verbosity == VERBOSE)
+	  say ("Patching file %s using Plan %s...\n",
+	       filename, using_plan_a ? "A" : "B");
+	else
+	  say ("patching file %s\n", filename);
       }
 }
 
 /* Report whether a desired revision was found.  */
 
 static void
-report_revision (found_revision)
-     int found_revision;
+report_revision (int found_revision)
 {
+  revision = quotearg (revision);
+
   if (found_revision)
     {
       if (verbosity == VERBOSE)
@@ -114,10 +118,8 @@ report_revision (found_revision)
 	     revision);
     }
   else if (batch)
-    {
-      fatal ("This file doesn't appear to be the %s version -- aborting.",
-	     revision);
-    }
+    fatal ("This file doesn't appear to be the %s version -- aborting.",
+	   revision);
   else
     {
       ask ("This file doesn't appear to be the %s version -- patch anyway? [n] ",
@@ -129,17 +131,14 @@ report_revision (found_revision)
 
 
 static void
-too_many_lines (filename)
-     char const *filename;
+too_many_lines (char const *filename)
 {
-  fatal ("File `%s' has too many lines.", filename);
+  fatal ("File %s has too many lines", quotearg (filename));
 }
 
 
 void
-get_input_file (filename, outname)
-     char const *filename;
-     char const *outname;
+get_input_file (char const *filename, char const *outname)
 {
     int elsewhere = strcmp (filename, outname);
     char const *cs;
@@ -168,18 +167,23 @@ get_input_file (filename, outname)
 		if (!elsewhere
 		    && (instat.st_mode & (S_IWUSR|S_IWGRP|S_IWOTH)) != 0)
 		    /* Somebody can write to it.  */
-		    fatal ("file `%s' seems to be locked by somebody else under %s",
-			   filename, cs);
-		/* It might be checked out unlocked.  See if it's safe to
-		   check out the default version locked.  */
-		if (verbosity == VERBOSE)
-		    say ("Comparing file `%s' to default %s version...\n",
-			 filename, cs);
-		if (systemic (diffbuf) != 0)
+		  fatal ("File %s seems to be locked by somebody else under %s",
+			 quotearg (filename), cs);
+		if (diffbuf)
 		  {
-		    say ("warning: patching file `%s', which does not match default %s version\n",
-			 filename, cs);
-		    cs = 0;
+		    /* It might be checked out unlocked.  See if it's safe to
+		       check out the default version locked.  */
+
+		    if (verbosity == VERBOSE)
+		      say ("Comparing file %s to default %s version...\n",
+			   quotearg (filename), cs);
+
+		    if (systemic (diffbuf) != 0)
+		      {
+			say ("warning: Patching file %s, which does not match default %s version\n",
+			     quotearg (filename), cs);
+			cs = 0;
+		      }
 		  }
 	    }
 
@@ -188,12 +192,13 @@ get_input_file (filename, outname)
 	      inerrno = 0;
 
 	    free (getbuf);
-	    free (diffbuf);
+	    if (diffbuf)
+	      free (diffbuf);
 
     } else if (inerrno && !pch_says_nonexistent (reverse))
       {
 	errno = inerrno;
-	pfatal ("can't find file `%s'", filename);
+	pfatal ("Can't find file %s", quotearg (filename));
       }
 
     if (inerrno)
@@ -202,15 +207,15 @@ get_input_file (filename, outname)
 	instat.st_size = 0;
       }
     else if (! S_ISREG (instat.st_mode))
-      fatal ("`%s' is not a regular file -- can't patch", filename);
+      fatal ("File %s is not a regular file -- can't patch",
+	     quotearg (filename));
 }
 
 
 /* Try keeping everything in memory. */
 
 static bool
-plan_a(filename)
-     char const *filename;
+plan_a (char const *filename)
 {
   register char const *s;
   register char const *lim;
@@ -232,7 +237,7 @@ plan_a(filename)
       int ifd = open (filename, O_RDONLY|binary_transput);
       size_t buffered = 0, n;
       if (ifd < 0)
-	pfatal ("can't open file `%s'", filename);
+	pfatal ("can't open file %s", quotearg (filename));
 
       while (size - buffered != 0)
 	{
@@ -315,8 +320,7 @@ plan_a(filename)
 /* Keep (virtually) nothing in memory. */
 
 static void
-plan_b(filename)
-     char const *filename;
+plan_b (char const *filename)
 {
   register FILE *ifp;
   register int c;
@@ -327,12 +331,15 @@ plan_b(filename)
   register char const *rev;
   register size_t revlen;
   register LINENUM line = 1;
+  int exclusive;
 
   if (instat.st_size == 0)
     filename = NULL_DEVICE;
   if (! (ifp = fopen (filename, binary_transput ? "rb" : "r")))
-    pfatal ("can't open file `%s'", filename);
-  tifd = create_file (TMPINNAME, O_RDWR | O_BINARY, (mode_t) 0);
+    pfatal ("Can't open file %s", quotearg (filename));
+  exclusive = TMPINNAME_needs_removal ? 0 : O_EXCL;
+  TMPINNAME_needs_removal = 1;
+  tifd = create_file (TMPINNAME, O_RDWR | O_BINARY | exclusive, (mode_t) 0);
   i = 0;
   len = 0;
   maxlen = 1;
@@ -415,13 +422,11 @@ plan_b(filename)
   input_lines = line - 1;
 }
 
-/* Fetch a line from the input file. */
+/* Fetch a line from the input file.
+   WHICHBUF is ignored when the file is in memory.  */
 
 char const *
-ifetch (line, whichbuf, psize)
-register LINENUM line;
-int whichbuf;				/* ignored when file in memory */
-size_t *psize;
+ifetch (LINENUM line, int whichbuf, size_t *psize)
 {
     register char const *q;
     register char const *p;
