@@ -1,5 +1,5 @@
 /* quotearg.c - quote arguments for output
-   Copyright (C) 1998 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,6 +17,8 @@
 
 /* Written by Paul Eggert <eggert@twinsun.com> */
 
+/* FIXME: Multibyte characters are not supported yet.  */
+
 #if HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -37,14 +39,21 @@
 # define ISGRAPH(c) (ISASCII (c) && isprint (c) && !isspace (c))
 #endif
 
+#if ENABLE_NLS
+# include <libintl.h>
+# define _(text) gettext (text)
+#else
+# define _(text) text
+#endif
+
 #if HAVE_LIMITS_H
 # include <limits.h>
 #endif
 #ifndef CHAR_BIT
-#define CHAR_BIT 8
+# define CHAR_BIT 8
 #endif
 #ifndef UCHAR_MAX
-#define UCHAR_MAX ((unsigned char) -1)
+# define UCHAR_MAX ((unsigned char) -1)
 #endif
 
 #if HAVE_STDLIB_H
@@ -61,8 +70,8 @@ struct quoting_options
 {
   /* Basic quoting style.  */
   enum quoting_style style;
-  
-  /* Quote the chararacters indicated by this bit vector even if the
+
+  /* Quote the characters indicated by this bit vector even if the
      quoting style would not normally require them to be quoted.  */
   int quote_these_too[((UCHAR_MAX + 1) / INT_BITS
 		       + ((UCHAR_MAX + 1) % INT_BITS != 0))];
@@ -71,7 +80,24 @@ struct quoting_options
 /* Names of quoting styles.  */
 char const *const quoting_style_args[] =
 {
-  "literal", "shell", "shell-always", "c", "escape", 0
+  "literal",
+  "shell",
+  "shell-always",
+  "c",
+  "escape",
+  "locale",
+  0
+};
+
+/* Correspondences to quoting style names.  */
+enum quoting_style const quoting_style_vals[] =
+{
+  literal_quoting_style,
+  shell_quoting_style,
+  shell_always_quoting_style,
+  c_quoting_style,
+  escape_quoting_style,
+  locale_quoting_style
 };
 
 /* The default quoting options.  */
@@ -119,7 +145,7 @@ set_char_quoting (struct quoting_options *o, char c, int i)
   *p ^= ((i & 1) ^ r) << shift;
   return r;
 }
-   
+
 /* Place into buffer BUFFER (of size BUFFERSIZE) a quoted version of
    argument ARG (of size ARGSIZE), using O to control quoting.
    If O is null, use the default.
@@ -130,16 +156,17 @@ set_char_quoting (struct quoting_options *o, char c, int i)
    If ARGSIZE is -1, use the string length of the argument for ARGSIZE.  */
 size_t
 quotearg_buffer (char *buffer, size_t buffersize,
-		 char const *arg, size_t argsize, 
+		 char const *arg, size_t argsize,
 		 struct quoting_options const *o)
 {
   unsigned char c;
   size_t i;
-  size_t len;
-  int quote_mark;
+  size_t len = 0;
+  char const *quote_string;
+  size_t quote_string_len;
   struct quoting_options const *p = o ? o : &default_quoting_options;
   enum quoting_style quoting_style = p->style;
-# define STORE(c) \
+#define STORE(c) \
     do \
       { \
 	if (len < buffersize) \
@@ -151,18 +178,17 @@ quotearg_buffer (char *buffer, size_t buffersize,
   switch (quoting_style)
     {
     case shell_quoting_style:
-      if (! (argsize == -1 ? arg[0] == '\0' : argsize == 0))
+      if (! (argsize == (size_t) -1 ? arg[0] == '\0' : argsize == 0))
 	{
 	  switch (arg[0])
 	    {
 	    case '#': case '~':
 	      break;
-	      
+
 	    default:
-	      len = 0;
 	      for (i = 0; ; i++)
 		{
-		  if (argsize == -1 ? arg[i] == '\0' : i == argsize)
+		  if (argsize == (size_t) -1 ? arg[i] == '\0' : i == argsize)
 		    goto done;
 
 		  c = arg[i];
@@ -184,32 +210,40 @@ quotearg_buffer (char *buffer, size_t buffersize,
 
 		  STORE (c);
 		}
-
 	    needs_quoting:;
+
+	      len = 0;
 	      break;
 	    }
 	}
       /* Fall through.  */
 
     case shell_always_quoting_style:
-      quote_mark = '\'';
+      STORE ('\'');
+      quote_string = "'";
+      quote_string_len = 1;
       break;
 
     case c_quoting_style:
-      quote_mark = '"';
+      STORE ('"');
+      quote_string = "\"";
+      quote_string_len = 1;
+      break;
+
+    case locale_quoting_style:
+      for (quote_string = _("`"); *quote_string; quote_string++)
+	STORE (*quote_string);
+      quote_string = _("'");
+      quote_string_len = strlen (quote_string);
       break;
 
     default:
-      quote_mark = 0;
+      quote_string = 0;
+      quote_string_len = 0;
       break;
     }
 
-  len = 0;
-
-  if (quote_mark)
-    STORE (quote_mark);
-
-  for (i = 0;  ! (argsize == -1 ? arg[i] == '\0' : i == argsize);  i++)
+  for (i = 0;  ! (argsize == (size_t) -1 ? arg[i] == '\0' : i == argsize);  i++)
     {
       c = arg[i];
 
@@ -230,6 +264,7 @@ quotearg_buffer (char *buffer, size_t buffersize,
 
 	case c_quoting_style:
 	case escape_quoting_style:
+	case locale_quoting_style:
 	  switch (c)
 	    {
 	    case '?': /* Do not generate trigraphs.  */
@@ -242,42 +277,38 @@ quotearg_buffer (char *buffer, size_t buffersize,
 	    case '\r': c = 'r'; goto store_escape;
 	    case '\t': c = 't'; goto store_escape;
 	    case '\v': c = 'v'; goto store_escape;
-	      
-	    case ' ':
-	      if (quoting_style == escape_quoting_style)
-		goto store_escape;
-	      break;
-	      
-	    case '"':
-	      if (quoting_style == c_quoting_style)
-		goto store_escape;
-	      break;
-	      
+
+	    case ' ': break;
+
 	    default:
+	      if (quote_string_len
+		  && strncmp (arg + i, quote_string, quote_string_len) == 0)
+		goto store_escape;
 	      if (!ISGRAPH (c))
 		{
 		  STORE ('\\');
 		  STORE ('0' + (c >> 6));
-		  STORE ('0' + ((c >> 3) & 3));
-		  c = '0' + (c & 3);
+		  STORE ('0' + ((c >> 3) & 7));
+		  c = '0' + (c & 7);
 		  goto store_c;
 		}
 	      break;
 	    }
-	  
+
 	  if (! (p->quote_these_too[c / INT_BITS] & (1 << (c % INT_BITS))))
 	    goto store_c;
-	  
+
 	store_escape:
 	  STORE ('\\');
 	}
-      
+
     store_c:
       STORE (c);
     }
 
-  if (quote_mark)
-    STORE (quote_mark);
+  if (quote_string)
+    for (; *quote_string; quote_string++)
+      STORE (*quote_string);
 
  done:
   if (len < buffersize)
@@ -289,11 +320,13 @@ quotearg_buffer (char *buffer, size_t buffersize,
    OPTIONS specifies the quoting options.
    The returned value points to static storage that can be
    reused by the next call to this function with the same value of N.
-   N must be nonnegative.  */
+   N must be nonnegative.  N is deliberately declared with type `int'
+   to allow for future extensions (using negative values).  */
 static char *
-quotearg_n_options (int n, char const *arg, struct quoting_options *options)
+quotearg_n_options (int n, char const *arg,
+		    struct quoting_options const *options)
 {
-  static unsigned nslots;
+  static unsigned int nslots;
   static struct slotvec
     {
       size_t size;
@@ -328,7 +361,7 @@ quotearg_n_options (int n, char const *arg, struct quoting_options *options)
 }
 
 char *
-quotearg_n (int n, char const *arg)
+quotearg_n (unsigned int n, char const *arg)
 {
   return quotearg_n_options (n, arg, &default_quoting_options);
 }
@@ -339,6 +372,21 @@ quotearg (char const *arg)
   return quotearg_n (0, arg);
 }
 
+char *
+quotearg_n_style (unsigned int n, enum quoting_style s, char const *arg)
+{
+  struct quoting_options o;
+  o.style = s;
+  memset (o.quote_these_too, 0, sizeof o.quote_these_too);
+  return quotearg_n_options (n, arg, &o);
+}
+
+char *
+quotearg_style (enum quoting_style s, char const *arg)
+{
+  return quotearg_n_style (0, s, arg);
+}
+
 char *
 quotearg_char (char const *arg, char ch)
 {
