@@ -1,6 +1,6 @@
 /* Print the version number.  */
 
-/* $Id: version.c,v 1.6 1998/03/15 14:44:47 eggert Exp $ */
+/* $Id: version.c,v 1.7 1999/08/30 06:20:08 eggert Exp $ */
 
 #define XTERN extern
 #include <common.h>
@@ -11,7 +11,7 @@
 
 static char const copyright_string[] = "\
 Copyright 1984-1988 Larry Wall\n\
-Copyright 1989-1998 Free Software Foundation, Inc.";
+Copyright 1989-1999 Free Software Foundation, Inc.";
 
 static char const free_software_msgid[] = "\
 This program comes with NO WARRANTY, to the extent permitted by law.\n\
@@ -20,10 +20,10 @@ under the terms of the GNU General Public License.\n\
 For more information about these matters, see the file named COPYING.";
 
 static char const authorship_msgid[] = "\
-written by Larry Wall with lots o' patches by Paul Eggert";
+written by Larry Wall and Paul Eggert";
 
 void
-version()
+version (void)
 {
   printf ("%s %s\n%s\n\n%s\n\n%s\n", program_name, PATCH_VERSION,
 	  copyright_string, free_software_msgid, authorship_msgid);
